/*  This file is part of the KDE project
 *  Copyright © 2010 Casian Andrei <skeletk13@gmail.com>
 *  Copyright © 2010 Harald Sitter <apachelogger@ubuntu.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), Nokia Corporation
 *  (or its successors, if any) and the KDE Free Qt Foundation, which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "objectdescription.h"

namespace Phonon
{

namespace Experimental
{

#ifndef PHONON_NO_VIDEOCAPTURE

VideoCaptureDevice phononVcdToExperimentalVcd(const Phonon::VideoCaptureDevice &vcd)
{
    QHash<QByteArray, QVariant> properties;
    const QList<QByteArray> &propertyNames = vcd.propertyNames();
    int pi, pn = propertyNames.count();
    for (pi = 0; pi < pn; ++ pi)
        properties[propertyNames[pi]] = vcd.property(propertyNames[pi].constData());

    return VideoCaptureDevice(vcd.index(), properties);
}

Phonon::VideoCaptureDevice phononExperimentalVcdToVcd(const Phonon::Experimental::VideoCaptureDevice &vcd)
{
    QHash<QByteArray, QVariant> properties;
    const QList<QByteArray> &propertyNames = vcd.propertyNames();
    int pi, pn = propertyNames.count();
    for (pi = 0; pi < pn; ++ pi)
        properties[propertyNames[pi]] = vcd.property(propertyNames[pi].constData());

    return Phonon::VideoCaptureDevice(vcd.index(), properties);
}

#endif // PHONON_NO_VIDEOCAPTURE

} // Experimental namespace
} // Phonon namespace
